/***************************************************************************
 *   Copyright (C) 2006 by Gustavo Pichorim Boiko   *
 *   boiko@mandriva.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "firstlevelmenu.h"

#include <qpainter.h>

#include <kdebug.h>
#include <kconfig.h>
#include <kservicegroup.h>

FirstLevelMenuItem::FirstLevelMenuItem(FirstLevelMenu *menu, Menu::Group *group)
: PixmapListViewItem(menu)
{
    m_relPath = group->relPath;
    setText(0, group->name);
}

FirstLevelMenuItem::FirstLevelMenuItem(FirstLevelMenu *menu, QListViewItem *after)
: PixmapListViewItem(menu, after)
{
}

void FirstLevelMenuItem::drawContents(QPainter *p, const QColorGroup &cg, int width, int height)
{
    /* copied from QListBoxText */
    QFontMetrics fm = p->fontMetrics();
    QFont font = p->font();
    font.setBold(isHover());
    p->setFont(font);
    int yPos = ((height - fm.height()) / 2) + fm.ascent();
    p->drawText( 3, yPos, text(0) );
}

FirstLevelMenu::FirstLevelMenu(KConfig *cfg, const QString &path, const QString &name)
 : PixmapListView(), ThemeObject(cfg, path, name)
{
    QPixmap pixmap(path + "/" + cfg->readEntry("ItemPixmap", QString::null));
    setItemPixmap(pixmap);
    
    pixmap = QPixmap(path + "/" + cfg->readEntry("ItemHoverPixmap", QString::null));
    setItemHoverPixmap(pixmap);
    
    pixmap = QPixmap(path + "/" + cfg->readEntry("BackgroundPixmap", QString::null));
    setBackgroundPixmap(pixmap);
    
    resize(cfg->readSizeEntry("Size", &QSize(100,100)));
    
}


FirstLevelMenu::~FirstLevelMenu()
{
}

void FirstLevelMenu::listServices(const Menu::MenuList& menu)
{
    clear();
    QDictIterator<Menu::Group> it(menu); // See QDictIterator
    for( ; it.current(); ++it )
    {
        (new FirstLevelMenuItem(this, it.current()));
    }
}

void FirstLevelMenu::slotClicked(QListViewItem *item)
{
    FirstLevelMenuItem *i = dynamic_cast<FirstLevelMenuItem*>(item);
    if (i)
    {
        emit serviceGroupSelected(i->relPath());
    }
}

void FirstLevelMenu::slotSearching(bool s)
{
    if (s == searching())
        return;
    
    setSearching(s);
    repaintContents(false);
}

void FirstLevelMenu::draw( QPainter *p, QRect rect)
{
    paintEmptyArea(p, rect);
}

#include "firstlevelmenu.moc"
