/***************************************************************************
 *   Copyright (C) 2006 by Gustavo Pichorim Boiko   *
 *   boiko@mandriva.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "menuapplet.h"
#include "pixmapbutton.h"
#include "menuwidget.h"

#include <kglobal.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kdebug.h>
#include <kpopupmenu.h>

#include <qlayout.h>
#include <qlabel.h>

extern "C"
{
    KPanelApplet* init(QWidget *parent, const QString& configFile)
    {
        KGlobal::locale()->insertCatalogue("mandrivamenu");
        return new MenuApplet(configFile, KPanelApplet::Normal,
                                 KPanelApplet::About | KPanelApplet::Preferences,
                                 parent, "mandrivamenu");
    }
}

MenuApplet::MenuApplet(const QString &configFile, Type t, int actions, QWidget *parent, 
                       const char *name, WFlags f)
: KPanelApplet(configFile, t, actions, parent, name, f)
{
    QHBoxLayout *layout = new QHBoxLayout(this);
    layout->setAutoAdd(true);
    m_button = new PixmapButton(this);
    KIconLoader *l = KGlobal::iconLoader();
    m_button->setNormalPixmap(l->loadIcon("menuk-mdk", KIcon::Panel, 32));
    m_button->setToggleBtn(true); 
    m_button->show();
    
    m_menu = new KPopupMenu(this);
    MenuWidget *w = new MenuWidget(m_menu);
    w->show();
    m_menu->setMask(w->mask());
    m_menu->insertItem(w);
    
    connect(m_button, SIGNAL(clicked()),
            this, SLOT(slotClicked()));
}


MenuApplet::~MenuApplet()
{
}

int MenuApplet::widthForHeight(int height) const
{
    return height;
}

int MenuApplet::heightForWidth(int width) const
{
    return width;
}

void MenuApplet::slotClicked()
{
    
    if (m_menu->isVisible())
        m_menu->hide();
    else
    {
        QPoint p;
        p = mapToGlobal(pos());
        p.setY(p.y() - m_menu->sizeHint().height());
        m_menu->exec(p);
    }
}

void MenuApplet::slotToggle(bool on)
{
    if (on)
    {
        QPoint p;
        p = mapToGlobal(pos());
        p.setY(p.y() - m_menu->sizeHint().height());
        m_menu->exec(p);
    }
    else
        m_menu->hide();
}


#include "menuapplet.moc"
