/***************************************************************************
 *   Copyright (C) 2006 by Gustavo Pichorim Boiko   *
 *   boiko@mandriva.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "menumanager.h"

#include <kglobal.h>
#include <kiconloader.h>
#include <kdebug.h>

#define ICON_SIZE 32

namespace Menu
{
    
Manager::Manager(QObject *parent, const char *name)
 : QObject(parent, name)
{
    m_list.setAutoDelete(true);
    buildMenu();
    m_lastShown = QString::null;
}


Manager::~Manager()
{
}

void Manager::buildMenu()
{
    KServiceGroup::Ptr root = KServiceGroup::root();
    
    if (!root || !root->isValid())
        return;
    KServiceGroup::List list = root->entries(true, true, true, false);
    
    KServiceGroup::List::ConstIterator it;
    for (it = list.begin(); it != list.end(); it++)
    {
        KSycocaEntry *s = (*it);
        if (s->isType(KST_KServiceGroup))
        {
            KServiceGroup *group = static_cast<KServiceGroup*>(s);
            if (group->childCount() > 0)
            {
                Group *grp = new Group;
                grp->entries.setAutoDelete(true);
                grp->name = group->caption();
                grp->relPath = group->relPath();
                fillEntries(grp, group);
                m_list.insert(grp->relPath, grp);
            }
        }
    }
}

void Manager::slotListMenu(const QString &menu)
{
    Group *group = m_list[menu];
    m_lastShown = menu;
    
    if (!group)
        return;
    
    emit listMenuEntries(group->entries);
}

void Manager::fillEntries( Group *menu, KServiceGroup *group )
{
    if (!group || !group->isValid())
        return;
    
    KServiceGroup::List list = group->entries(true, true, true, false);
    KServiceGroup::List::ConstIterator it;
    for (it = list.begin(); it != list.end(); it++)
    {
        KSycocaEntry *s = (*it);
        if (s->isType(KST_KService))
        {
            KService *service = static_cast<KService*>(s);
            Entry *entry = new Entry;
            
            QString icon = service->icon();
            entry->icon = KGlobal::iconLoader()->loadIcon(icon, KIcon::Desktop, ICON_SIZE);
            entry->name = service->name();
            entry->genericName = service->genericName();
            entry->comment = service->comment();
            entry->exec = service->exec();
            entry->path = service->desktopEntryPath();
            menu->entries.insert(entry->name, entry);
        }
    }
}

void Manager::clear()
{
    // FIXME: check wheter second level lists are deleted
    m_list.clear();
}

void Manager::slotSearch(const QString &text)
{
    
    // Just for demo purposes
    EntryList list;
    list.setAutoDelete(false);
    
    if (text.length() < 3)
    {
        emit searching(false);
        if (m_lastShown != QString::null)
            slotListMenu(m_lastShown);
        else
            emit listMenuEntries(list);
        return;
    }
    emit searching(true);
    QDictIterator<Group> it(m_list); // See QDictIterator
    for( ; it.current(); ++it )
    {
        QDictIterator<Entry> entry(it.current()->entries);
        for (; entry.current(); ++entry)
        {
            Entry *e = entry.current();
            if (e->name.contains(text, false) || 
                e->genericName.contains(text,false) ||
                e->comment.contains(text,false))
            {
                list.insert(e->name, e);
            }   
        }
    }
    //if (list.count() > 0)
        emit listMenuEntries(list);
}

}
#include "menumanager.moc"
