/***************************************************************************
 *   Copyright (C) 2006 by Gustavo Pichorim Boiko   *
 *   boiko@mandriva.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MENUMANAGER_H
#define MENUMANAGER_H

#include <qobject.h>
#include <qvaluelist.h>
#include <qdict.h>
#include <qpixmap.h>

#include <kservicegroup.h>

namespace Menu
{
    
typedef struct 
{
    QString name;
    QString comment;
    QString genericName;
    QString exec;
    QString path;
    QPixmap icon;
} Entry;
typedef QDict<Entry> EntryList;

typedef struct 
{
    QString name;
    QString relPath;
    EntryList entries;
} Group;

typedef QDict<Group> MenuList;

/**
A class to manage menu entries

	@author Gustavo Pichorim Boiko <boiko@mandriva.com>
*/
class Manager : public QObject
{
Q_OBJECT
public:
    Manager(QObject *parent = 0, const char *name = 0);

    ~Manager();
    
    MenuList rootMenu() const { return m_list; }

private:
    void buildMenu();
    void fillEntries(Group *menu, KServiceGroup *group);
    void clear();
    MenuList m_list;
    QString m_lastShown;
    
signals:
    void listMenuEntries(const EntryList& list);
    void searching(bool s);
    
public slots:
    void slotListMenu(const QString &menu);
    void slotSearch(const QString &text);
    
};

}

#endif
