/***************************************************************************
 *   Copyright (C) 2006 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "pixmapbutton.h"

#include <qpainter.h>

#include <kdebug.h>
#include <kiconeffect.h>
#include <kicontheme.h>
#include <kconfig.h>

PixmapButton::PixmapButton(QWidget *parent, const char *name)
 : QButton(parent, name)
{
    m_hover = false;
    connect(this, SIGNAL(clicked()),
            this, SLOT(slotClicked()));
    
    setToggleButton(false);
}


PixmapButton::~PixmapButton()
{
}

void PixmapButton::setHoverPixmap(const QPixmap &pixmap)
{
    m_hoverPixmap = pixmap;
    if (m_hover)
        repaint();
}

void PixmapButton::setNormalPixmap(const QPixmap &pixmap)
{
    m_pixmap = pixmap;
    resize(pixmap.size());
}

void PixmapButton::setDownPixmap(const QPixmap &pixmap)
{
    m_downPixmap = pixmap;
    if (isDown())
        repaint();
}

void PixmapButton::drawButton(QPainter *p)
{
    QPixmap bg(size());
    if (backgroundPixmap())
    {
        bg = *backgroundPixmap();
    }
    else
    {
        QWidget *w = parentWidget();
        if (w)
            bg.fill(w, rect().x(), rect().y());
    }
    QPoint point(0,0);
    p->drawPixmap(point, bg);
    
    if (isDown())
    {
        QPixmap pix;
        KIconEffect fx;
        if (!m_downPixmap.isNull())
            pix = m_downPixmap;
        else if (!m_hoverPixmap.isNull())
            pix =  fx.apply(m_hoverPixmap, KIconEffect::ToGray, 0.5, Qt::white, false);
        else if (!m_pixmap.isNull())
            pix = fx.apply(m_pixmap, KIconEffect::ToGray, 0.5, Qt::white, false);
        
        if (!pix.isNull())
        {
            p->drawPixmap(point, pix);
        }
    }
    else if (m_hover)
    {
        if (!m_hoverPixmap.isNull())
            p->drawPixmap(point, m_hoverPixmap);
        else if (!m_pixmap.isNull())
        {
            KIconEffect fx;
            p->drawPixmap(point, fx.apply(m_pixmap, KIcon::Desktop, KIcon::ActiveState)); 
        }
    }
    else if (!m_pixmap.isNull())
        p->drawPixmap(point, m_pixmap);
     
}

void PixmapButton::drawButtonLabel(QPainter *p)
{
    //do nothing
}

void PixmapButton::enterEvent(QEvent *e)
{
    m_hover = true;
    repaint();
}

void PixmapButton::leaveEvent(QEvent *e)
{
    m_hover = false;
    repaint();
}

bool PixmapButton::authorized()
{
    // well, in the default implementation we can always show the button
    return true;
}

void PixmapButton::slotClicked()
{
    // do nothing for now
}

void PixmapButton::resizeEvent(QResizeEvent *e)
{
    QImage img;
    if (!m_pixmap.isNull())
    {
        // this costs, but it is executed very few times, so, not much a problem
        img = m_pixmap.convertToImage();
        m_pixmap.convertFromImage(img.smoothScale(e->size()));
    }
    if (!m_hoverPixmap.isNull())
    {
        img = m_hoverPixmap.convertToImage();
        m_hoverPixmap.convertFromImage(img.smoothScale(e->size()));
    }
    if (!m_downPixmap.isNull())
    {
        img = m_downPixmap.convertToImage();
        m_downPixmap.convertFromImage(img.smoothScale(e->size()));
    }
    
    repaint();
}

void PixmapButton::setToggleBtn(bool t)
{
    setToggleButton(t); 
}

#include "pixmapbutton.moc"
