/***************************************************************************
 *   Copyright (C) 2006 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <qpainter.h>
#include <qheader.h> 
#include <qcursor.h>

#include <kdebug.h>
#include <kiconeffect.h>
#include <kicontheme.h>

#include "pixmaplistview.h"
PixmapListViewItem::PixmapListViewItem(PixmapListView *lv)
: QListViewItem(lv)
{
    m_hover = false;
}

PixmapListViewItem::PixmapListViewItem(PixmapListView *lv, QListViewItem *after)
: QListViewItem(lv, after)
{
}

PixmapListViewItem::~PixmapListViewItem()
{
}

void PixmapListViewItem::paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int align)
{
    if (column != 0)
        return;
    
    PixmapListView *lv = dynamic_cast<PixmapListView*>(listView());
    if (lv)
    {
        QRect rect(0, 0, width, height());
        QPixmap pix(rect.size());
        QPainter painter(&pix);
        painter.eraseRect(rect);
        
        if (m_hover)
            painter.drawPixmap(rect, lv->itemHoverPixmap());
        else if (lv->searching())
            painter.drawPixmap(rect, lv->disabledPixmap());
        else if (isSelected())
            painter.drawPixmap(rect, lv->itemHoverPixmap());
        else
            painter.drawPixmap(rect, lv->itemPixmap());
        /* ask subclasses to draw contents */
        drawContents(&painter, cg, width, rect.height());
        painter.flush();
        
        // draw offscreen pixmap
        p->drawPixmap(rect.topLeft(), pix);
    }
}

void PixmapListViewItem::drawContents(QPainter *p, const QColorGroup &cg, int width, int height)
{
    //do nothing in the default implementation
}

void PixmapListViewItem::setup()
{
    PixmapListView *lv = dynamic_cast<PixmapListView*>(listView());
    if (lv)
        setHeight(lv->itemPixmap().height());
}

PixmapListView::PixmapListView(QWidget *parent, const char *name)
 : KListView(parent, name)
{
    m_hover = 0;
    m_searching = false;
    setFrameStyle(NoFrame);
    setVScrollBarMode(QScrollView::AlwaysOff);
    addColumn("Items", -1);
    header()->setStretchEnabled(true, 0);
    header()->hide();
    
    connect(this, SIGNAL(clicked(QListViewItem*)),
            this, SLOT(slotClicked(QListViewItem*)));
    
    connect(this, SIGNAL(onItem(QListViewItem*)),
            this, SLOT(slotOnItem(QListViewItem*)));
    
    connect(&m_scrollTimer, SIGNAL(timeout()),
            this, SLOT(slotScroll()));
}


PixmapListView::~PixmapListView()
{
}

void PixmapListView::setItemPixmap(const QPixmap &pixmap)
{
    m_itemPixmap = pixmap;
    
    // generate a disabled pixmap (a grayed pixmap)
    KIconEffect fx;
    
    m_disabledPixmap = fx.apply(pixmap, KIconEffect::ToGray, 0.7, Qt::gray, false);
    repaint();
}

void PixmapListView::setItemHoverPixmap(const QPixmap &pixmap)
{
    QImage img = pixmap.convertToImage();
    if (!m_itemPixmap.isNull())
    {
        if (m_itemPixmap.width() != pixmap.width() || 
            m_itemPixmap.height() != pixmap.height())
        {
            img.smoothScale(m_itemPixmap.width(), m_itemPixmap.height());
        }
    }
    m_itemHoverPixmap.convertFromImage(img);
    repaint();
}

void PixmapListView::setBackgroundPixmap(const QPixmap &pixmap)
{
    m_backgroundPixmap = pixmap;
    repaint();
}

void PixmapListView::slotClicked(QListViewItem *item)
{
    Q_UNUSED(item);
    // do nothing in the default implementation
}

void PixmapListView::slotOnItem(QListViewItem *i)
{
    if (m_hover)
    {
        m_hover->setHover(false);
        repaintItem(m_hover);
    }
    PixmapListViewItem *item = dynamic_cast<PixmapListViewItem*>(i);
    
    if (!item)
        return;
    
    item->setHover(true);
    repaintItem(item);
    m_hover = item;
}
void PixmapListView::contentsMouseMoveEvent(QMouseEvent *e)
{
    QPoint pos = contentsToViewport(e->pos());
    
#define DELTA 5
    if (pos.y() > viewport()->width() - DELTA)
    {
        m_scrollUp = false;
        if (!m_scrollTimer.isActive())
        {
            m_interval = 50;
            m_scrollTimer.start(50);
        }
    }
    else if (pos.y() < DELTA)
    {
        m_scrollUp = true;
        if (!m_scrollTimer.isActive())
        {
            m_interval = 50;
            m_scrollTimer.start(50);
        }
    }
    else if (m_scrollTimer.isActive())
    {
        m_scrollTimer.stop();
    }
    KListView::contentsMouseMoveEvent(e);
}

void PixmapListView::leaveEvent(QEvent *e)
{
    if (m_hover)
    {
        m_hover->setHover(false);
        repaintItem(m_hover);
    }
    m_hover = 0;
    if (m_scrollTimer.isActive())
        m_scrollTimer.stop();
}

void PixmapListView::paintEmptyArea(QPainter *p, const QRect &rect)
{
    QPixmap pix(rect.size());
    
    if (m_backgroundPixmap.isNull())
    {
        QWidget *w = parentWidget(true);
        
        if (!w)
        {
            KListView::paintEmptyArea(p,rect);
            return;
        }
        pix.fill(w, mapTo(w, rect.topLeft()));
        p->drawPixmap(rect, pix);
        return;
    }
    
    if (m_backgroundPixmap.rect().contains(rect))
    {
        p->drawPixmap(rect.topLeft(), m_backgroundPixmap, rect);
    }
}

void PixmapListView::slotScroll()
{
    QScrollBar *bar = verticalScrollBar();
    if (m_scrollUp)
    {
        if (bar->value() == bar->minValue())
        {
            m_scrollTimer.stop();
            return;
        }
        bar->setValue(bar->value()-1);
    }
    else
    {
        if (bar->value() == bar->maxValue())
        {
            m_scrollTimer.stop();
            return;
        }
        bar->setValue(bar->value()+1);
    }
    if (m_interval > 5)
    {
        m_scrollTimer.stop();
        m_scrollTimer.start(--m_interval);
    }
    QMouseEvent e(QEvent::MouseMove, mapFromGlobal(QCursor::pos()), QCursor::pos(), 0, 0);
    
    KListView::contentsMouseMoveEvent(&e);
}

/*!
  Creates a string with ... like "Trollte..." or "...olltech" depending on the alignment

  taken from qlistview.cpp
 */

QString qEllipsisText(const QString &org, const QFontMetrics &fm, int width, int align)
{
    int ellWidth = fm.width( "..." );
    QString text = QString::fromLatin1("");
    int i = 0;
    int len = org.length();
    int offset = (align & Qt::AlignRight) ? (len-1) - i : i;
    
    if (fm.width(org) <= width)
        return org;
    
    while ( i < len && fm.width( text + org[ offset ] ) + ellWidth < width ) {
        if ( align & Qt::AlignRight )
            text.prepend( org[ offset ] );
        else
            text += org[ offset ];
        offset = (align & Qt::AlignRight) ? (len-1) - ++i : ++i;
    }
    if ( text.isEmpty() )
        text = ( align & Qt::AlignRight ) ? org.right( 1 ) : text = org.left( 1 );
    if ( align & Qt::AlignRight )
        text.prepend( "..." );
    else
        text += "...";
    return text;
}
#include "pixmaplistview.moc"
