/***************************************************************************
 *   Copyright (C) 2006 by Gustavo Pichorim Boiko   *
 *   boiko@mandriva.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PIXMAPLISTVIEW_H
#define PIXMAPLISTVIEW_H

#include <klistview.h>
#include <qpixmap.h>
#include <qtimer.h>

class PixmapListView;

class PixmapListViewItem : public QListViewItem
{
public:
    PixmapListViewItem(PixmapListView *lv);
    PixmapListViewItem(PixmapListView *lv, QListViewItem *after);
    ~PixmapListViewItem();
        
    void setHover(bool h) { m_hover = h; }
    bool isHover() { return m_hover; }
    void setTruncated(bool t) { m_truncated = t; }
    bool isTruncated() { return m_truncated; }
    virtual void setup();

protected:
    virtual void paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int align);
    virtual void drawContents(QPainter *p, const QColorGroup &cg, int width, int height);

private:
    QString m_relPath;
    bool m_hover;
    bool m_truncated;
};

/**
A listview that accepts pixmaps for drawing items

	@author Gustavo Pichorim Boiko <boiko@mandriva.com>
*/
class PixmapListView : public KListView
{
Q_OBJECT
public:
    PixmapListView(QWidget *parent = 0, const char *name = 0);

    ~PixmapListView();
    
    void setItemPixmap(const QPixmap &pixmap);
    void setItemHoverPixmap(const QPixmap &pixmap);
    void setBackgroundPixmap(const QPixmap &pixmap);
    
    QPixmap itemPixmap() { return m_itemPixmap; }
    QPixmap disabledPixmap() { return m_disabledPixmap; }
    QPixmap itemHoverPixmap() { return m_itemHoverPixmap; }
    QPixmap backgroundPixmap() { return m_backgroundPixmap; }
    bool searching() { return m_searching; }
    void setSearching(bool s) { m_searching = s; }
    
protected:
    virtual void leaveEvent(QEvent *e);
    virtual void paintEmptyArea(QPainter *p, const QRect &rect);
    
private:
    bool m_searching;
    QPixmap m_itemPixmap;
    QPixmap m_disabledPixmap;
    QPixmap m_itemHoverPixmap;
    QPixmap m_backgroundPixmap;
    PixmapListViewItem *m_hover;
    
    // scrolling stuff
    QTimer m_scrollTimer;
    bool m_scrollUp;
    int m_interval;
    
protected slots:
    virtual void slotClicked(QListViewItem *item);
    virtual void contentsMouseMoveEvent(QMouseEvent *e);
            
private slots:
    void slotOnItem(QListViewItem *i);
    
    void slotScroll();
};

QString qEllipsisText(const QString &org, const QFontMetrics &fm, int width, int align);

#endif
