/***************************************************************************
 *   Copyright (C) 2006 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "secondlevelmenu.h"

#include <qpainter.h>
#include <qheader.h>

#include <kconfig.h>
#include <kservice.h>
#include <kservicegroup.h>
#include <kdebug.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <dcopref.h>
#include <kurl.h>
#include <kurldrag.h>

// FIXME: choose the icon size according to the menu height
#define ICON_SIZE 32

SecondLevelMenuItem::SecondLevelMenuItem(SecondLevelMenu *menu, Menu::Entry *entry)
: PixmapListViewItem(menu)
{
    m_entry = entry;
    setText(0, entry->name);
}

SecondLevelMenuItem::SecondLevelMenuItem(SecondLevelMenu *menu, QListViewItem *after)
: PixmapListViewItem(menu, after)
{
}

void SecondLevelMenuItem::drawContents(QPainter *p, const QColorGroup &cg, int width, int height)
{
    QPixmap ico;
    // draw icon
    ico = icon();
    
    if (isHover() && !ico.isNull())
        ico = ico.convertToImage().smoothScale(ico.width() + 2, ico.height()+2);
    
    bool trunc = false;
    
    QString txt;
    QString txtTruncated;
    int top = (height - ico.height()) / 2;
    p->drawPixmap( QPoint(3, top), ico);
    /* copied from QListBoxText */
    QFont font = p->font();
    font.setBold(isHover());
    p->setFont(font);
    QFontMetrics fm = p->fontMetrics();
    int yPos = ( ( height - (2*fm.height()) ) / 2 ) + fm.ascent();
    txtTruncated = qEllipsisText(text(0), fm, width - 10 - ico.width(), Qt::AlignLeft);
    trunc = (txtTruncated != text(0));
    p->drawText( 10 + ico.width(), yPos, txtTruncated );
    
    // Draw a separator line
    p->setPen(QColor(250,181,87));
    p->drawLine(10 + ico.width(), height / 2, width - 10, height /2);
    // draw the description or generic name
    if (!genericName().isEmpty())
        txt = genericName();
    else
        txt = comment();
    if (!isHover())
        p->setPen(Qt::gray);
    else
        p->setPen(Qt::red);
    yPos = ( ( height/2 - (fm.height()) ) / 2 ) + fm.ascent() + height/2;
    trunc = trunc && (txtTruncated != txt);
    setTruncated(trunc);
    txtTruncated = qEllipsisText(txt, fm, width - 10 - ico.width(), Qt::AlignLeft); 
    p->drawText( 10 + ico.width(), yPos, txtTruncated);
    
    // draw a rectangle all around the item
    p->setPen(Qt::black);
    p->drawRect(0,0,width,height);
}

QString SecondLevelMenuItem::name()
{
    if (m_entry)
        return m_entry->name;
}

QString SecondLevelMenuItem::comment()
{
    if (m_entry)
        return m_entry->comment;
}

QString SecondLevelMenuItem::genericName()
{
    if (m_entry)
        return m_entry->genericName;
}

QString SecondLevelMenuItem::exec()
{
    if (m_entry)
        return m_entry->exec;
}

QString SecondLevelMenuItem::path()
{
    if (m_entry)
        return m_entry->path;
}

QPixmap SecondLevelMenuItem::icon() const
{
    if (m_entry)
        return m_entry->icon;
}

SecondLevelMenu::SecondLevelMenu(KConfig *cfg, const QString &path, const QString &name)
 : PixmapListView(), ThemeObject(cfg, path, name)
{
    QPixmap pixmap(path + "/" + cfg->readEntry("ItemPixmap", QString::null));
    setItemPixmap(pixmap);
    
    pixmap = QPixmap(path + "/" + cfg->readEntry("ItemHoverPixmap", QString::null));
    setItemHoverPixmap(pixmap);
    
    pixmap = QPixmap(path + "/" + cfg->readEntry("BackgroundPixmap", QString::null));
    setBackgroundPixmap(pixmap);
    
    setSelectionMode(QListView::NoSelection);
    resize(cfg->readSizeEntry("Size", &QSize(100,100)));
}

SecondLevelMenu::~SecondLevelMenu()
{
}

void SecondLevelMenu::listMenuEntries(const Menu::EntryList& list)
{
    clear();
    
    QDictIterator<Menu::Entry> it(list); // See QDictIterator
    for( ; it.current(); ++it )
        (new SecondLevelMenuItem(this, it.current()));
    
}

void SecondLevelMenu::slotClicked(QListViewItem *i)
{
    SecondLevelMenuItem *item = dynamic_cast<SecondLevelMenuItem*>(i);
    
    if (!item)
        return;
    
    DCOPRef ref("klauncher", "default");
    
    ref.send("start_service_by_desktop_name", item->name(), QStringList());
}

QDragObject *SecondLevelMenu::dragObject()
{
    if (!currentItem())
        return 0;
    QListViewItem *itm = currentItem();
    SecondLevelMenuItem *item = dynamic_cast<SecondLevelMenuItem*>(itm);
    if (!item)
        return 0;
            
    KURL::List urls;
    urls.append(KURL(item->path()));
    KURLDrag* drag = new KURLDrag(urls, this, item->name());
    drag->setPixmap(item->icon());
    
    return drag;
}

void SecondLevelMenu::draw(QPainter *p, QRect rect)
{
    paintEmptyArea(p, rect);
}
#include "secondlevelmenu.moc"
