/***************************************************************************
 *   Copyright (C) 2006 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "theme.h"
#include "themeobject.h"
#include "button.h"
#include "image.h"
#include "firstlevelmenu.h"
#include "secondlevelmenu.h"
#include "searchbar.h"
#include "userface.h"
#include "text.h"

#include <kconfig.h>
#include <kdebug.h>
#include <kapplication.h>
#include <dcopclient.h>

#include <qpixmap.h>
#include <qpainter.h>


namespace Theme
{
    ThemeObject *createObject(KConfig *cfg, const QString &path, const QString &objName)
    {
        cfg->setGroup(objName);
        QString type = cfg->readEntry("Type");
        
        if (type == "Button")
        {
            Button *btn = new Button(cfg, path, objName);
            return btn;
        }
        else if (type == "Image")
        {
            Image *img = new Image(cfg, path, objName);
            return img;
        }
        else if (type == "UserFace")
        {
            UserFace *face = new UserFace(cfg, path, objName);
            return face;
        }
        else if (type == "Text")
        {
            Text *text = new Text(cfg, path, objName);
            return text;
        }
        else if (type =="FirstLevelMenu")
        {
            FirstLevelMenu *menu = new FirstLevelMenu(cfg, path, objName);
            return menu;
        }
        else if (type == "SecondLevelMenu")
        {
            SecondLevelMenu *menu = new SecondLevelMenu(cfg, path, objName);
            return menu;
        }
        else if (type == "SearchBar")
        {
            SearchBar *bar = new SearchBar(cfg, path, objName);
            return bar;
        }
        else
        {
            kdDebug() << "mandrivamenu: Unknown object type: " << type << endl;
            return 0;
        }
            
    }
    
    ThemeObject *findObject(const QDict<ThemeObject> &list, ObjectType type)
    {
        QDictIterator<ThemeObject> it(list);
        for (; it.current(); ++it)
        {
            if (it.current()->objectType() == type)
                return it.current();
        }
        return 0;
    }
    
    void execAction(Action action)
    {
        // for simple actions, put the code in this function
        // for more complex ones, create a separate function
        if (action == LockAction)
        {
            // Copied from kicker lockout applet
            QCString appname( "kdesktop" );
            int kicker_screen_number = qt_xscreen();
            if ( kicker_screen_number )
                appname.sprintf("kdesktop-screen-%d", kicker_screen_number);
            kapp->dcopClient()->send(appname, "KScreensaverIface", "lock()", "");
        }
        else if (action == LogoutAction)
        {
            kapp->requestShutDown();
        }
    }
    bool actionAuthorized(Action action)
    {
        if (action == LockAction)
            return kapp->authorize("lock_screen");
        else if (action == LogoutAction)
            return kapp->authorize("logout");
        else
            return true;
    }
    
    QPixmap createBackground(const QDict<ThemeObject> &list, const QRect &rect, QWidget *root)
    {
        QPixmap pix(rect.size());
        pix.fill(root, rect.topLeft());
        QWidget *w;
        QDictIterator<ThemeObject> it(list);
        for (; it.current(); ++it)
        {
            w = dynamic_cast<QWidget*>(it.current());
            
            if (!w)
                continue;
            
            QRect r = w->rect();
            
            r.moveTopLeft(it.current()->position());
            if (r.intersects(rect))
            {
                QRect inter = r & rect;
                
                QRect src, dst;
                src = dst = inter;
                
                src.moveLeft(inter.x() - r.x());
                src.moveTop(inter.y() - r.y());
                
                dst.moveLeft(inter.x() - rect.x());
                dst.moveTop(inter.y() - rect.y());

                QPainter p(&pix);
                p.translate((dst.x() - src.x()), (dst.y() - src.y()));
                it.current()->draw(&p, src); 
                bitBlt(&pix, dst.topLeft(), w, src);
            }
        }
        return pix;
    }
}


