/***************************************************************************
 *   Copyright (C) 2006 by Gustavo Pichorim Boiko                          *
 *   boiko@mandriva.com                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef THEME_H
#define THEME_H

#include <qstring.h>
#include <qdict.h>
#include <qpixmap.h>

class ThemeObject;
class KConfig;

namespace Theme 
{
    
    enum ObjectType 
    {
        ButtonObj,
        ImageObj,
        UserFaceObj,
        TextObj,
        FirstLevelMenuObj,
        SecondLevelMenuObj,
        SearchBarObj
    };

    enum Action
    {
        NoAction,
        LockAction,
        LogoutAction
    };

    /**
     * Creates a theme object based on a KConfig file entry
     */
    ThemeObject *createObject(KConfig *cfg, const QString &path, const QString &objName);
    
    /**
     * Returns the first object of the specified type
     */
    ThemeObject *findObject(const QDict<ThemeObject> &list, ObjectType type); 
    
    /**
     * An action launcher
     */
    void execAction(Action action);
    
    /**
     * Checks wheter the action is authorized
     */
    bool actionAuthorized(Action action);
    
    QPixmap createBackground(const QDict<ThemeObject> &list, const QRect &rect, QWidget *root);
}
#endif
