/***************************************************************************
 *   Copyright (C) 2006 by Gustavo Pichorim Boiko   *
 *   boiko@mandriva.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "userface.h"

#include <kimageeffect.h>
#include <kconfig.h>
#include <kuser.h>
#include <kstandarddirs.h>
#include <kdebug.h>

#include <qpainter.h>
UserFace::UserFace(KConfig *cfg, const QString &path, const QString &name)
 : PixmapWidget(), ThemeObject(cfg, path, name)
{
    
    // user face
    QImage img(path + "/" + cfg->readEntry("UserPixmap", QString::null));
    if (img.isNull())
    {
        // get .face icon
        KUser user;
        img.load(locate("data", user.homeDir() + "/.face.icon"));
    }
    setUserPixmap(img);
    
    //box image
    QImage foobar(path + "/" + cfg->readEntry("BoxPixmap", QString::null));
    setBoxPixmap(foobar);
}

UserFace::~UserFace()
{
}

void UserFace::setUserPixmap(const QImage &pixmap)
{
    if (!pixmap.isNull())
    {
        m_userImage = pixmap;
    }
    
    updateUserFace(m_userImage, m_boxImage);
}

void UserFace::setBoxPixmap(const QImage &pixmap)
{
    if (!pixmap.isNull())
    {
        m_boxImage = pixmap;
    }
    
    updateUserFace(m_userImage, m_boxImage);
}

void UserFace::updateUserFace(const QImage &user, const QImage &box)
{
    QImage boxImg;
    QImage userImg = user;
    QImage target;
    
    if (box.isNull())
        boxImg = user;
    else if (!user.isNull())
    {
        boxImg = box;
        QPoint p;
        
        if (user.width() > box.width() - 4 || user.height() > box.height() - 4)
        {
            userImg = user.smoothScale((box.size() - QSize(4,4)), QImage::ScaleMin);
        }
         
        
        int x,y;
        x = (box.width() - userImg.width())/2;
        y = (box.height() - userImg.height())/2;
        
        bitBlt(&boxImg, x, y, &userImg, 0, 0, userImg.width(), userImg.height());
    }
    
    QPixmap pix;
    if (!boxImg.isNull())
        pix.convertFromImage(boxImg);
    
    setPixmap(pix);
}

void UserFace::draw(QPainter *p, QRect rect)
{
    paint(p, rect);
}

#include "userface.moc"
